@@ -17,11 +17,10 @@ from bytecode import UNSET, BasicBlock, Bytecode, Compare, ControlFlowGraph, Ins
 from networkx import lowest_common_ancestor
 from networkx.drawing.nx_pydot import to_pydot
 
-# Key for storing branch value in networkx edge.
-from ordered_set import OrderedSet
-
 import pynguin.utils.opcodes as op
+from pynguin.utils.orderedset import OrderedSet
 
+# Key for storing branch value in networkx edge.
 EDGE_DATA_BRANCH_VALUE = "branch_value"
 
 
