@@ -8,30 +8,30 @@
 from __future__ import annotations
 
 import logging
-from typing import TYPE_CHECKING, Any, cast
-
-from ordered_set import OrderedSet
-from typing_inspect import get_args, get_origin
+from typing import TYPE_CHECKING, cast
 
 import pynguin.configuration as config
 import pynguin.testcase.statement as stmt
 import pynguin.utils.generic.genericaccessibleobject as gao
 from pynguin.analyses.constants import ConstantProvider, EmptyConstantProvider
+from pynguin.analyses.typesystem import (
+    InferredSignature,
+    Instance,
+    NoneType,
+    ProperType,
+    TupleType,
+    is_collection_type,
+    is_primitive_type,
+)
 from pynguin.utils import randomness
 from pynguin.utils.exceptions import ConstructionFailedException
-from pynguin.utils.type_utils import (
-    is_collection_type,
-    is_consistent_with,
-    is_optional_parameter,
-    is_primitive_type,
-    is_type_unknown,
-)
+from pynguin.utils.orderedset import OrderedSet
+from pynguin.utils.type_utils import is_optional_parameter
 
 if TYPE_CHECKING:
     import pynguin.testcase.testcase as tc
     import pynguin.testcase.variablereference as vr
     from pynguin.analyses.module import ModuleTestCluster
-    from pynguin.analyses.typesystem import InferredSignature
 
 
 # TODO(fk) find better name for this?
@@ -67,7 +67,7 @@ class TestFactory:
             statement: The statement to append
             position: The position to insert the statement, default is at the end
                 of the test case
-            allow_none: Whether or not parameter variables can hold None values
+            allow_none: Whether parameter variables can hold None values
 
         Raises:
             ConstructionFailedException: if construction of an object failed
@@ -122,7 +122,7 @@ class TestFactory:
             position: The position to insert the statement, default is at the end
                 of the test case
             recursion_depth: The recursion depth for search
-            allow_none: Whether or not parameter variables can hold None values
+            allow_none: Whether parameter variables can hold None values
 
         Returns:
             An optional variable reference to the added statement
@@ -192,7 +192,7 @@ class TestFactory:
             position: The position where to put the statement in the test case,
                 defaults to the end of the test case
             recursion_depth: A recursion limit for the search of parameter values
-            allow_none: Whether or not a variable can be an None value
+            allow_none: Whether a variable can be a None value
 
         Returns:
             A variable reference to the constructor
@@ -254,7 +254,7 @@ class TestFactory:
             position: The position where to put the statement in the test case,
                 defaults to the end of the test case
             recursion_depth: A recursion limit for the search of parameter values
-            allow_none: Whether or not a variable can hold a None value
+            allow_none: Whether a variable can hold a None value
             callee: The callee, if it is already known.
 
         Returns:
@@ -275,7 +275,11 @@ class TestFactory:
         length = test_case.size()
         if callee is None:
             callee = self._create_or_reuse_variable(
-                test_case, method.owner, position, recursion_depth, allow_none=False
+                test_case,
+                self._test_cluster.type_system.make_instance(method.owner),
+                position,
+                recursion_depth,
+                allow_none=False,
             )
         assert callee, "The callee must not be None"
         parameters: dict[str, vr.VariableReference] = self.satisfy_parameters(
@@ -336,7 +340,11 @@ class TestFactory:
         length = test_case.size()
         if callee is None:
             callee = self._create_or_reuse_variable(
-                test_case, field.owner, position, recursion_depth, allow_none=False
+                test_case,
+                self._test_cluster.type_system.make_instance(field.owner),
+                position,
+                recursion_depth,
+                allow_none=False,
             )
         assert callee, "The callee must not be None"
         position = position + test_case.size() - length
@@ -401,7 +409,7 @@ class TestFactory:
             position: the position where to put the statement in the test case,
                 defaults to the end of the test case
             recursion_depth: A recursion limit for the search of parameter values
-            allow_none: Whether or not a variable can hold a None value
+            allow_none: Whether a variable can hold a None value
 
         Returns:
             A variable reference to the function call's result
@@ -503,7 +511,7 @@ class TestFactory:
             position: The last position before that the call is inserted
 
         Returns:
-            Whether or not the insertion was successful
+            Whether the insertion was successful
         """
         variable = self._select_random_variable_for_call(test_case, position)
         success = False
@@ -527,9 +535,8 @@ class TestFactory:
             position: The last position before that the call is inserted
 
         Returns:
-            Whether or not the insertion was successful
+            Whether the insertion was successful
         """
-        assert variable.type, "Cannot insert random call on variable of unknown type."
         try:
             accessible = self._test_cluster.get_random_call_for(variable.type)
             return self.add_call_for(test_case, variable, accessible, position)
@@ -553,7 +560,7 @@ class TestFactory:
             position: The last position
 
         Returns:
-            Whether or not the insertion was successful
+            Whether the insertion was successful
 
         Raises:
             RuntimeError: in case of an unknown accessible
@@ -592,7 +599,6 @@ class TestFactory:
             var
             for var in test_case.get_all_objects(position)
             if not var.is_primitive()
-            and not var.is_type_unknown()
             and not isinstance(
                 test_case.get_statement(var.get_statement_position()),
                 stmt.NoneStatement,
@@ -612,7 +618,7 @@ class TestFactory:
             position: The position of the insertion
 
         Returns:
-            Whether or not the insertion was successful
+            Whether the insertion was successful
         """
         previous_length = test_case.size()
         accessible = self._test_cluster.get_random_accessible()
@@ -654,7 +660,7 @@ class TestFactory:
             position: The position
 
         Returns:
-            Whether or not the deletion was successful
+            Whether the deletion was successful
         """
         variable = test_case.get_statement(position).ret_val
 
@@ -685,7 +691,7 @@ class TestFactory:
             position: The position
 
         Returns:
-            Whether or not the deletion was successful
+            Whether the deletion was successful
         """
         to_delete: set[int] = set()
         TestFactory._recursive_delete_inclusion(test_case, to_delete, position)
@@ -733,15 +739,13 @@ class TestFactory:
             statement: The new statement
 
         Returns:
-            Whether or not the operation was successful
+            Whether the operation was successful
         """
-        if statement.ret_val.is_type_unknown():
-            return False
-
         objects = test_case.get_all_objects(statement.get_position())
         type_ = statement.ret_val.type
-        assert type_, "Cannot change change call, when type is unknown"
-        calls = self._get_possible_calls(type_, objects)
+        # We need a consistent signature, otherwise nothing will match up
+        signature_memo: dict[InferredSignature, dict[str, ProperType]] = {}
+        calls = self._get_possible_calls(type_, objects, signature_memo)
         acc_object = statement.accessible_object()
         if acc_object in calls:
             calls.remove(acc_object)
@@ -751,7 +755,7 @@ class TestFactory:
 
         call = randomness.choice(calls)
         try:
-            self.change_call(test_case, statement, call)
+            self.change_call(test_case, statement, call, signature_memo)
             return True
         except ConstructionFailedException:
             self._logger.debug("Failed to change call for statement.", exc_info=True)
@@ -762,6 +766,7 @@ class TestFactory:
         test_case: tc.TestCase,
         statement: stmt.VariableCreatingStatement,
         call: gao.GenericAccessibleObject,
+        signature_memo: dict[InferredSignature, dict[str, ProperType]],
     ):
         """Change the call of the given statement to the one that is given.
 
@@ -769,6 +774,7 @@ class TestFactory:
             test_case: The test case
             statement: The given statement
             call: The new call
+            signature_memo: a memo to remember the chosen parameter types.
 
         Raises:
             AssertionError: when an unhandled call is encountered.
@@ -779,21 +785,23 @@ class TestFactory:
         if call.is_method():
             method = cast(gao.GenericMethod, call)
             assert method.owner
-            callee = self._get_random_non_none_object(test_case, method.owner, position)
+            callee = test_case.get_random_object(
+                self._test_cluster.type_system.make_instance(method.owner), position
+            )
             parameters = self._get_reuse_parameters(
-                test_case, method.inferred_signature, position
+                test_case, method.inferred_signature, position, signature_memo
             )
             replacement = stmt.MethodStatement(test_case, method, callee, parameters)
         elif call.is_constructor():
             constructor = cast(gao.GenericConstructor, call)
             parameters = self._get_reuse_parameters(
-                test_case, constructor.inferred_signature, position
+                test_case, constructor.inferred_signature, position, signature_memo
             )
             replacement = stmt.ConstructorStatement(test_case, constructor, parameters)
         elif call.is_function():
             funktion = cast(gao.GenericFunction, call)
             parameters = self._get_reuse_parameters(
-                test_case, funktion.inferred_signature, position
+                test_case, funktion.inferred_signature, position, signature_memo
             )
             replacement = stmt.FunctionStatement(test_case, funktion, parameters)
         elif call.is_enum():
@@ -808,7 +816,10 @@ class TestFactory:
 
     @staticmethod
     def _get_reuse_parameters(
-        test_case: tc.TestCase, inf_signature: InferredSignature, position: int
+        test_case: tc.TestCase,
+        inf_signature: InferredSignature,
+        position: int,
+        signature_memo: dict[InferredSignature, dict[str, ProperType]],
     ) -> dict[str, vr.VariableReference]:
         """Find specified parameters from existing objects.
 
@@ -816,12 +827,15 @@ class TestFactory:
             test_case: The test case
             inf_signature: The inferred signature information
             position: The position
+            signature_memo: a memo to remember the chosen parameter types.
 
         Returns:
             A dict of existing objects
         """
         found = {}
-        for parameter_name, parameter_type in inf_signature.parameters.items():
+        for parameter_name, parameter_type in inf_signature.get_parameter_types(
+            signature_memo
+        ).items():
             if (
                 is_optional_parameter(inf_signature, parameter_name)
                 and randomness.next_float()
@@ -833,27 +847,11 @@ class TestFactory:
             )
         return found
 
-    @staticmethod
-    def _get_random_non_none_object(
-        test_case: tc.TestCase, type_: type, position: int
-    ) -> vr.VariableReference:
-        variables = test_case.get_objects(type_, position)
-        variables = [
-            var
-            for var in variables
-            if not isinstance(
-                test_case.get_statement(var.get_statement_position()),
-                stmt.NoneStatement,
-            )
-        ]
-        if len(variables) == 0:
-            raise ConstructionFailedException(
-                f"Found no variables of type {type_} at position {position}"
-            )
-        return randomness.choice(variables)
-
     def _get_possible_calls(
-        self, return_type: type, objects: list[vr.VariableReference]
+        self,
+        return_type: ProperType,
+        objects: list[vr.VariableReference],
+        signature_memo: dict[InferredSignature, dict[str, ProperType]],
     ) -> list[gao.GenericAccessibleObject]:
         """Retrieve all the replacement calls that can be inserted at this position
          without changing the length.
@@ -861,23 +859,22 @@ class TestFactory:
          Args:
             return_type: The return type
             objects: The objects that are available as parameters.
+            signature_memo: a memo to remember the chosen parameter types.
 
         Returns:
             A list of possible replacement calls
         """
         calls: list[gao.GenericAccessibleObject] = []
-        try:
-            all_calls = self._test_cluster.get_generators_for(return_type)
-        except ConstructionFailedException:
-            return calls
+        all_calls, _ = self._test_cluster.get_generators_for(return_type)
         for i in all_calls:
-            if self._dependencies_satisfied(i.get_dependencies(), objects):
+            if self._dependencies_satisfied(
+                i.get_dependencies(signature_memo), objects
+            ):
                 calls.append(i)
         return calls
 
-    @staticmethod
     def _dependencies_satisfied(
-        dependencies: set[type], objects: list[vr.VariableReference]
+        self, dependencies: OrderedSet[ProperType], objects: list[vr.VariableReference]
     ) -> bool:
         """Determine if the set of objects is sufficient to satisfy the set of
         dependencies.
@@ -887,13 +884,14 @@ class TestFactory:
             objects: A list of objects
 
         Returns:
-            Whether or not the objects are sufficient to satisfy the dependencies
+            Whether the objects are sufficient to satisfy the dependencies
         """
         for type_ in dependencies:
             found = False
             for var in objects:
-                if is_consistent_with(var.type, type_):
+                if self._test_cluster.type_system.is_maybe_subtype(var.type, type_):
                     found = True
+                    break
             if not found:
                 return False
         return True
@@ -903,23 +901,18 @@ class TestFactory:
         self,
         test_case: tc.TestCase,
         signature: InferredSignature,
-        callee: vr.VariableReference | None = None,
         position: int = -1,
         recursion_depth: int = 0,
         allow_none: bool = True,
-        can_reuse_existing_variables: bool = True,
     ) -> dict[str, vr.VariableReference]:
         """Satisfy a list of parameters by reusing or creating variables.
 
         Args:
             test_case: The test case
             signature: The inferred signature of the method
-            callee: The callee of the method
             position: The current position in the test case
             recursion_depth: The recursion depth
-            allow_none: Whether or not a variable can be a None value
-            can_reuse_existing_variables: Whether or not existing variables shall
-                be reused.
+            allow_none: Whether a variable can be a None value
 
         Returns:
             A dict of variable references for the parameters
@@ -933,11 +926,11 @@ class TestFactory:
         parameters: dict[str, vr.VariableReference] = {}
         self._logger.debug(
             "Trying to satisfy %d parameters at position %d",
-            len(signature.parameters),
+            len(signature.original_parameters),
             position,
         )
 
-        for parameter_name, parameter_type in signature.parameters.items():
+        for parameter_name, parameter_type in signature.get_parameter_types({}).items():
             self._logger.debug("Current parameter type: %s", parameter_type)
 
             previous_length = test_case.size()
@@ -949,28 +942,14 @@ class TestFactory:
             ):
                 continue
 
-            if can_reuse_existing_variables:
-                self._logger.debug("Can re-use variables")
-                var = self._create_or_reuse_variable(
-                    test_case,
-                    parameter_type,
-                    position,
-                    recursion_depth,
-                    allow_none,
-                    callee,
-                )
-            else:
-                self._logger.debug(
-                    "Cannot re-use variables: attempt to creating new one"
-                )
-                var = self._create_variable(
-                    test_case,
-                    parameter_type,
-                    position,
-                    recursion_depth,
-                    allow_none,
-                    callee,
-                )
+            var = self._create_or_reuse_variable(
+                test_case,
+                parameter_type,
+                position,
+                recursion_depth,
+                allow_none,
+            )
+
             if not var:
                 raise ConstructionFailedException(
                     (
@@ -987,7 +966,7 @@ class TestFactory:
         return parameters
 
     def _reuse_variable(
-        self, test_case: tc.TestCase, parameter_type: type | None, position: int
+        self, test_case: tc.TestCase, parameter_type: ProperType, position: int
     ) -> vr.VariableReference | None:
         """Reuse an existing variable, if possible.
 
@@ -1001,9 +980,9 @@ class TestFactory:
         """
 
         objects = test_case.get_objects(parameter_type, position)
-        probability = (
+        probability: float = (
             config.configuration.test_creation.primitive_reuse_probability
-            if is_primitive_type(parameter_type)
+            if parameter_type.accept(is_primitive_type)
             else config.configuration.test_creation.object_reuse_probability
         )
         if objects and randomness.next_float() <= probability:
@@ -1015,7 +994,7 @@ class TestFactory:
     def _get_variable_fallback(
         self,
         test_case: tc.TestCase,
-        parameter_type: type | None,
+        parameter_type: ProperType,
         position: int,
         recursion_depth: int,
         allow_none: bool,
@@ -1027,7 +1006,7 @@ class TestFactory:
             parameter_type: the type of the variable that is needed
             position: the position to limit the search
             recursion_depth: the current recursion level
-            allow_none: whether or not a None value is allowed
+            allow_none: whether a None value is allowed
 
         Returns:
             A variable if found
@@ -1039,17 +1018,18 @@ class TestFactory:
 
         # No objects to choose from, so either create random type variable or use None.
         if not objects:
-            if (
-                config.configuration.type_inference.guess_unknown_types
-                and randomness.next_float() <= 0.85
-            ):
-                return self._create_random_type_variable(
-                    test_case, position, recursion_depth, allow_none
+            if randomness.next_float() <= 0.85:
+                return self._create_or_reuse_variable(
+                    test_case=test_case,
+                    parameter_type=randomness.choice(
+                        self._test_cluster.get_all_generatable_types()
+                    ),
+                    position=position,
+                    recursion_depth=recursion_depth + 1,
+                    allow_none=allow_none,
                 )
             if allow_none:
-                return self._create_none(
-                    test_case, parameter_type, position, recursion_depth
-                )
+                return self._create_none(test_case, position, recursion_depth)
             raise ConstructionFailedException(f"No objects for type {parameter_type}")
 
         # Could not create, so re-use an existing variable.
@@ -1066,32 +1046,22 @@ class TestFactory:
     def _create_or_reuse_variable(
         self,
         test_case: tc.TestCase,
-        parameter_type: type | None,
+        parameter_type: ProperType,
         position: int,
         recursion_depth: int,
         allow_none: bool,
-        exclude: vr.VariableReference | None = None,
     ) -> vr.VariableReference | None:
-        if is_type_unknown(parameter_type):
-            if config.configuration.type_inference.guess_unknown_types:
-                parameter_type = randomness.choice(
-                    self._test_cluster.get_all_generatable_types()
-                )
-            else:
-                return None
-
         if (
             reused_variable := self._reuse_variable(test_case, parameter_type, position)
         ) is not None:
             return reused_variable
         if (
-            created_variable := self._create_variable(
+            created_variable := self._attempt_generation(
                 test_case,
                 parameter_type,
                 position,
                 recursion_depth,
                 allow_none,
-                exclude,
             )
         ) is not None:
             return created_variable
@@ -1099,138 +1069,108 @@ class TestFactory:
             test_case, parameter_type, position, recursion_depth, allow_none
         )
 
-    # pylint: disable=too-many-arguments
-    def _create_variable(
-        self,
-        test_case: tc.TestCase,
-        parameter_type: type | None,
-        position: int,
-        recursion_depth: int,
-        allow_none: bool,
-        exclude: vr.VariableReference | None = None,
-    ) -> vr.VariableReference | None:
-        return self._attempt_generation(
-            test_case, parameter_type, position, recursion_depth, allow_none, exclude
-        )
-
     # pylint: disable=too-many-arguments
     def _attempt_generation(
         self,
         test_case: tc.TestCase,
-        parameter_type: type | None,
+        parameter_type: ProperType,
         position: int,
         recursion_depth: int,
         allow_none: bool,
-        exclude: vr.VariableReference | None = None,
     ) -> vr.VariableReference | None:
         # We only select a concrete type e.g. from a union, when we are forced to
         # choose one.
         parameter_type = self._test_cluster.select_concrete_type(parameter_type)
 
-        if not parameter_type:
-            return None
-
-        if (
-            allow_none
-            and randomness.next_float()
-            <= config.configuration.test_creation.none_probability
-        ):
-            return self._create_none(
-                test_case, parameter_type, position, recursion_depth
-            )
-        if is_primitive_type(parameter_type):
+        if isinstance(parameter_type, NoneType):
+            return self._create_none(test_case, position, recursion_depth)
+        if parameter_type.accept(is_primitive_type):
             return self._create_primitive(
                 test_case,
-                parameter_type,
+                cast(Instance, parameter_type),
                 position,
                 recursion_depth,
                 constant_provider=self._constant_provider,
             )
-        if is_collection_type(parameter_type):
+        if parameter_type.accept(is_collection_type):
             return self._create_collection(
                 test_case,
                 parameter_type,
                 position,
                 recursion_depth,
             )
-        if type_generators := self._test_cluster.get_generators_for(parameter_type):
-            return self._attempt_generation_for_type(
-                test_case, position, recursion_depth, allow_none, type_generators
+        type_generators, only_any = self._test_cluster.get_generators_for(
+            parameter_type
+        )
+        if type_generators and not only_any:
+            type_generator = randomness.choice(type_generators)
+            return self.append_generic_accessible(
+                test_case,
+                type_generator,
+                position=position,
+                recursion_depth=recursion_depth + 1,
+                allow_none=allow_none,
             )
         return None
 
-    def _attempt_generation_for_type(
-        self,
-        test_case: tc.TestCase,
-        position: int,
-        recursion_depth: int,
-        allow_none: bool,
-        type_generators: OrderedSet[gao.GenericAccessibleObject],
-    ) -> vr.VariableReference | None:
-        type_generator = randomness.choice(type_generators)
-        return self.append_generic_accessible(
-            test_case,
-            type_generator,
-            position=position,
-            recursion_depth=recursion_depth + 1,
-            allow_none=allow_none,
-        )
-
-    def _create_random_type_variable(
-        self,
-        test_case: tc.TestCase,
-        position: int,
-        recursion_depth: int,
-        allow_none: bool,
-    ) -> vr.VariableReference | None:
-        return self._create_or_reuse_variable(
-            test_case=test_case,
-            parameter_type=randomness.choice(
-                self._test_cluster.get_all_generatable_types()
-            ),
-            position=position,
-            recursion_depth=recursion_depth + 1,
-            allow_none=allow_none,
-        )
-
     @staticmethod
     def _create_none(
         test_case: tc.TestCase,
-        parameter_type: type | None,
         position: int,
         recursion_depth: int,
     ) -> vr.VariableReference:
-        statement = stmt.NoneStatement(test_case, parameter_type)
+        # If there already is a None alias just return it.
+        # TODO(fk) better way?
+        for statement in test_case.statements[
+            : min(len(test_case.statements), position)
+        ]:
+            if isinstance(statement, stmt.NoneStatement):
+                return statement.ret_val
+
+        statement = stmt.NoneStatement(test_case)
         ret = test_case.add_variable_creating_statement(statement, position)
         ret.distance = recursion_depth
         return ret
 
-    @staticmethod
     def _create_primitive(
+        self,
         test_case: tc.TestCase,
-        parameter_type: type,
+        parameter_type: Instance,
         position: int,
         recursion_depth: int,
         constant_provider: ConstantProvider,
     ) -> vr.VariableReference:
-        if parameter_type == int:
-            statement: stmt.PrimitiveStatement = stmt.IntPrimitiveStatement(
-                test_case, constant_provider=constant_provider
-            )
-        elif parameter_type == float:
-            statement = stmt.FloatPrimitiveStatement(
-                test_case, constant_provider=constant_provider
-            )
-        elif parameter_type == bool:
-            statement = stmt.BooleanPrimitiveStatement(test_case)
-        elif parameter_type == bytes:
-            statement = stmt.BytesPrimitiveStatement(
-                test_case, constant_provider=constant_provider
-            )
-        else:
-            statement = stmt.StringPrimitiveStatement(
-                test_case, constant_provider=constant_provider
-            )
+        # Need to adhere to numeric tower.
+        if (
+            subtypes := self._test_cluster.type_system.numeric_tower.get(parameter_type)
+        ) is not None:
+            parameter_type = randomness.choice(subtypes)
+
+        match parameter_type.type.name:
+            case "int":
+                statement: stmt.PrimitiveStatement = stmt.IntPrimitiveStatement(
+                    test_case, constant_provider=constant_provider
+                )
+            case "float":
+                statement = stmt.FloatPrimitiveStatement(
+                    test_case, constant_provider=constant_provider
+                )
+            case "complex":
+                statement = stmt.ComplexPrimitiveStatement(
+                    test_case, constant_provider=constant_provider
+                )
+            case "bool":
+                statement = stmt.BooleanPrimitiveStatement(test_case)
+            case "bytes":
+                statement = stmt.BytesPrimitiveStatement(
+                    test_case, constant_provider=constant_provider
+                )
+            case "str":
+                statement = stmt.StringPrimitiveStatement(
+                    test_case, constant_provider=constant_provider
+                )
+            case _:
+                raise RuntimeError(f"Unknown primitive {parameter_type}")
         ret = test_case.add_variable_creating_statement(statement, position)
         ret.distance = recursion_depth
         return ret
@@ -1238,38 +1178,36 @@ class TestFactory:
     def _create_collection(
         self,
         test_case: tc.TestCase,
-        parameter_type: type,
+        parameter_type: ProperType,
         position: int,
         recursion_depth: int,
     ) -> vr.VariableReference:
-        if parameter_type in (list, set) or get_origin(parameter_type) in (list, set):
-            return self._create_list_or_set(
-                test_case, parameter_type, position, recursion_depth
-            )
-        if parameter_type == tuple or get_origin(parameter_type) == tuple:
+        if isinstance(parameter_type, Instance):
+            if parameter_type.type.raw_type in (list, set):
+                return self._create_list_or_set(
+                    test_case, parameter_type, position, recursion_depth
+                )
+            if parameter_type.type.raw_type == dict:
+                return self._create_dict(
+                    test_case, parameter_type, position, recursion_depth
+                )
+        if isinstance(parameter_type, TupleType):
             return self._create_tuple(
                 test_case, parameter_type, position, recursion_depth
             )
-        if parameter_type == dict or get_origin(parameter_type) == dict:
-            return self._create_dict(
-                test_case, parameter_type, position, recursion_depth
-            )
         raise RuntimeError("Unknown collection type")
 
     # TODO(fk) Methods below should be refactored asap,
-    # as they contain a lot of duplicate code
+    #  as they contain a lot of duplicate code.
+    # TODO(fk) improve generic support.
     def _create_list_or_set(
         self,
         test_case: tc.TestCase,
-        parameter_type: type,
+        parameter_type: Instance,
         position: int,
         recursion_depth: int,
     ) -> vr.VariableReference:
-        args = get_args(parameter_type)
-        if len(args) == 1:
-            element_type = args[0]
-        else:
-            element_type = Any
+        element_type = parameter_type.args[0]
         size = randomness.next_int(
             0, config.configuration.test_creation.collection_size
         )
@@ -1284,7 +1222,7 @@ class TestFactory:
             position += test_case.size() - previous_length
         collection_stmt = (
             stmt.ListStatement(test_case, parameter_type, elements)
-            if parameter_type == list or get_origin(parameter_type) == list
+            if parameter_type.type.raw_type == list
             else stmt.SetStatement(test_case, parameter_type, elements)
         )
         ret = test_case.add_variable_creating_statement(collection_stmt, position)
@@ -1294,20 +1232,21 @@ class TestFactory:
     def _create_tuple(
         self,
         test_case: tc.TestCase,
-        parameter_type: type,
+        parameter_type: TupleType,
         position: int,
         recursion_depth: int,
     ) -> vr.VariableReference:
-        args = get_args(parameter_type)
-        if len(args) == 0:
+        if parameter_type.unknown_size:
             # Untyped tuple, time to guess...
             size = randomness.next_int(
                 0, config.configuration.test_creation.collection_size
             )
-            args = [
+            args = tuple(
                 randomness.choice(self._test_cluster.get_all_generatable_types())
                 for _ in range(size)
-            ]
+            )
+        else:
+            args = parameter_type.args
         elements = []
         for arg_type in args:
             previous_length = test_case.size()
@@ -1326,17 +1265,13 @@ class TestFactory:
     def _create_dict(
         self,
         test_case: tc.TestCase,
-        parameter_type: type,
+        parameter_type: Instance,
         position: int,
         recursion_depth: int,
     ) -> vr.VariableReference:
-        args = get_args(parameter_type)
-        if len(args) == 2:
-            key_type = args[0]
-            value_type = args[1]
-        else:
-            key_type = Any
-            value_type = Any
+        args = parameter_type.args
+        key_type = args[0]
+        value_type = args[1]
         size = randomness.next_int(
             0, config.configuration.test_creation.collection_size
         )
@@ -1368,7 +1303,7 @@ class TestFactory:
             test_case: the test case to check.
 
         Returns:
-            True, iff the test case has a call on the SUT.
+            True, if the test case has a call on the SUT.
         """
         for statement in test_case.statements:
             if (
